from .components import Slot, Template
from .content import DD, DL, DT, HR, OL, UL, BlockQuote, Div, FigCaption, Figure, P, Pre
from .forms import (
    URL,
    Button,
    Checkbox,
    Color,
    DataList,
    Date,
    DatetimeLocal,
    Email,
    FieldSet,
    File,
    Form,
    Hidden,
    Image,
    Input,
    Label,
    Legend,
    Meter,
    Month,
    Number,
    OptGroup,
    Option,
    Output,
    Password,
    Progress,
    Radio,
    Range,
    Reset,
    Search,
    Select,
    Submit,
    Tel,
    Text,
    TextArea,
    TimeInput,
    Week,
)
from .interactive_elements import Details, Dialog, Menu, Summary
from .media import SVG, Area, Audio, Img, Map, Path, Polygon, Track, Video
from .metadata import Base, Link, Meta, Style, Title
from .scripts import Canvas, NoScript, Script
from .sectioning import (
    H1,
    H2,
    H3,
    H4,
    H5,
    H6,
    HTML,
    Address,
    Article,
    Aside,
    Body,
    Footer,
    Head,
    Header,
    HGroup,
    Main,
    Nav,
    Section,
)
from .semantics import (
    BDI,
    BDO,
    RP,
    RT,
    A,
    Abbr,
    B,
    Br,
    Cite,
    Code,
    Data,
    Del,
    Dfn,
    Em,
    I,
    Ins,
    Kbd,
    Mark,
    Q,
    Ruby,
    S,
    Samp,
    Small,
    Span,
    Strong,
    Sub,
    Sup,
    Time,
    U,
    Var,
    WBr,
)
from .tables import TD, TH, TR, Caption, Col, ColGroup, Table, TBody, TFoot, THead
